#!/usr/bin/env bash


##############################################################################################
#                              Method Utils Functions                                        #
#                                                                                            #
# Functions to facilitate running of methods scripts as part of an omnibenchmark.            #
# Most functions are specifically target omnibenchmark scripts and are only valid or useful  #
# in an omnibenchmark context/repository.                                                    #
#                                                                                            #
# Change History                                                                             #
# 10/06/2021  Almut Lütge    Original code: functions to run omnibenchmark methods while     #
#                            exploring their parameter space                                 #
#                                                                                            #
# author: Almut Lütge                                                                        #
##############################################################################################
##############################################################################################
##############################################################################################


##############################################################################################

## ---------------------------------------------------------------------------------------- ##
## ------------------------------------- Prerequisites  ------------------------------------##
## ---------------------------------------------------------------------------------------- ##

{ # try
    source src/utils/bash/dataset_utils.sh
} || { 
    echo "Warning, utils/bash/dataset_utils.sh does not exist and may be required for some functions."
}

{ # try
    source src/config.sh
} || { 
    echo "Warning, src/config.sh does not exist and may be required for some functions."
}

# Error function
error_exit()
{
  printf '%s\n' "$@" 1>&2
  exit 1
} 


##############################################################################################

## ---------------------------------------------------------------------------------------- ##
## ------------------------------- Help Documentations  ------------------------------------##
## ---------------------------------------------------------------------------------------- ##

help_find_all_param_json()
{
   echo "Function to find and check the json file to define the overall parameter space. Returns the json file's path."
   echo
   echo "Syntax: find_all_param_json PARAM_KEY"
   echo
   echo "Arguments:"
   echo "PARAM_KEY      Name of the parameter dataset with a json file defining the overall parameter space. This dataset needs to be part of the repository."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_get_method_param_combs()
{
   echo "Function to generate a list of all valid parameter combinations for a method."
   echo
   echo "Syntax: get_method_param_combs PARAM_JSON PARAM_AR"
   echo
   echo "Arguments:"
   echo "PARAM_JSON     Json file with path, that defines the overall parameter space for this benchmark."
   echo "PARAM_AR       Name of the parameter array to define all valid parameter (keys) for this method with their default values (default PARAM_AR as defined in src/config.sh)."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_generate_workflow_json()
{
   echo "Function to generate a json file from all methods specific valid parameter combinations to define workflow settings as used as input for renku workflow loop."
   echo
   echo "Syntax: generate_workflow_json PARAM_LIST"
   echo
   echo "Arguments:"
   echo "PARAM_LIST     List with all valid parameter combinations as generated by get_method_param_combs. See get_method_param_combs -h for help."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}



##############################################################################################

function dataset_exist () {

    # Input: Dataset_name
    # Output: Dataset name, if dataset exist, else an empty string
    
    local data_name=$1
    renku dataset ls --format json | jq -r '.[] | .name' | grep -E -x "$data_name"   
}


## ---------------------------------------------------------------------------------------- ##
## --------------------- Functions to access parameter space  ------------------------------##
## ---------------------------------------------------------------------------------------- ##


# ---------------------------- find_all_param_json ----------------------------------------- #

function find_all_param_json (){

    # Input1: PARAM_KEY the keyword of the dataset that describes the overall parameter space
    # Output: Json file that defines the overall parameter space. Must be imported into the project.
    
    # Display help message
    local OPTIND
    while getopts ":h" option; do
        case $option in
            h) 
                help_find_all_param_json
                return 1;;
            \?) 
                error_exit "Error: Invalid option";;
            esac
    done
    shift $((OPTIND-1))
    
    # Define variables and generate output
    local param_key=$1
    local param_name
    param_name=$(query_datasets_by_keyword "$param_key" name | sort | uniq)
    
    if [[ -z $(dataset_exist "$param_name") ]]
    then
        error_exit "Dataset $param_name not found." \
                   "Please check PARAM_KEY in config.sh and/or import $param_name."
    fi
    
    local param_json
    param_json=$(renku dataset ls-files "${param_name}" -c path | grep json)
       
    # Check output is only one file
    local file_length
    file_length=$(echo "${param_json}" | wc -w)
    
    case $file_length in
            0) 
                error_exit "ERROR: Parameter file does not exist. Please check parameter dataset." \
                            "Detected: ${param_json}";;
            1)
                echo "${param_json}";;
            
            \?) 
                error_exit "ERROR: Multiple json files are found as part of the parameter dataset." \
                           "Make sure the dataset contains only one json file." \
                           "The following files were detected:" \
                           "${param_json}";;
            esac
}


# ------------------------------- get_all_param_combs --------------------------------- #


function get_method_param_combs (){
    
    # Input1: Path to the parameter.json file that defines the overall parameter space for a benchmark.
    # Input2: Name of the parameter array, that defines the valid parameter for a specific method.
    # Output: List with all valid parameter combinations
    
    # Display help message
    local OPTIND
    while getopts ":h" option; do
        case $option in
            h) 
                help_get_method_param_combs
                return 1;;
            \?) 
                error_exit "Error: Invalid option";;
            esac
    done
    shift $((OPTIND-1))
    
    # Define variables
    local param_json=$1
    local -n param_ar=${2:-PARAM_AR}
    local var_all=()
    local param_keys=("${!param_ar[@]}")
    
    # Check if all parameter are defined in parmeter json
    json_keys=$(cat "$param_json" | jq -r 'keys | .[]')
    
    # Generate all valid parameter combinations 
    for param in "${!param_ar[@]}"; do
        
        if [[ ! "${json_keys[@]}" =~ (^|[[:space:]])$param($|[[:space:]]) ]];
        then
            error_exit "Could not find values for parameter $param." \
                       "Please make sure $param and all possible values are specified in $param_json." \
                       "As $param_json was imported this should be done in this benchmarks parameter repo."
        fi
    
        if [[ $param == "$param_keys" ]];
        then
            var_print=$(cat "${param_json}" | jq -r ".${param} | .[]" | sed 's/ /'",${param}'='"'/g' | sed 's/^/'"$param'='"'/')
        else
            var_print=$(cat "${param_json}" | jq -r ".${param} | .[]" | sed 's/ /'",_$param'='"'/g' | sed 's/^/'"_$param'='"'/')
        fi
        var_all+="{${var_print}}"
    
    done

    eval $(echo echo "${var_all[@]}")
}


# --------------------------- generate_workflow_json ---------------------------------- #

function generate_workflow_json (){
    # input parameter combinations as from get_method_param_combs
    # output workflow description json to use with renku workflow loop
    
    # Display help message
    local OPTIND
    while getopts ":h" option; do
        case $option in
            h) 
                help_parse_default_params_to_Rargs
                return 1;;
            \?) 
                error_exit "Error: Invalid option";;
            esac
    done
    shift $((OPTIND-1))
    
    # Define inputs
    local param_all="$*"
    local json_key="valid_combinations"
    
    # Generate output
    key_val=$(echo {\""$param_all"\"} | sed "s/ /\" }, { \"/g" | sed "s/=/\": \"/g" | sed "s/_/\", \"/g") 
    
    echo -e "{ \"$json_key\": [$key_val]}\n"   
}


##############################################################################################
